using Geekeey.Extensions.Process.Buffered;
using Geekeey.Extensions.Process.Tests.Dummy;

namespace Geekeey.Extensions.Process.Tests;

[TestFixture]
internal sealed class BufferedExecuteTests
{
	[Test]
	public async Task ExecuteBuffered_Stdout()
	{
		// Arrange
		var cmd = new Command(Program.FilePath)
			.WithArguments(["echo", "--target", "stdout", "Hello stdout",]);

		// Act
		var result = await cmd.ExecuteBufferedAsync();

		Assume.That(result.ExitCode, Is.EqualTo(0));

		// Assert
		Assert.Multiple(() =>
		{
			Assert.That(result.StandardOutput.Trim(), Is.EqualTo("Hello stdout"));
			Assert.That(result.StandardError.Trim(), Is.Empty);
		});
	}

	[Test]
	public async Task ExecuteBuffered_Stderr()
	{
		// Arrange
		var cmd = new Command(Program.FilePath)
			.WithArguments(["echo", "--target", "stderr", "Hello stdout",]);

		// Act
		var result = await cmd.ExecuteBufferedAsync();

		Assume.That(result.ExitCode, Is.EqualTo(0));

		// Assert
		Assert.Multiple(() =>
		{
			Assert.That(result.StandardOutput.Trim(), Is.Empty);
			Assert.That(result.StandardError.Trim(), Is.EqualTo("Hello stdout"));
		});
	}

	[Test]
	public async Task ExecuteBuffered_Stdout_And_Stderr()
	{
		// Arrange
		var cmd = new Command(Program.FilePath)
			.WithArguments(["echo", "--target", "all", "Hello stdout and stderr",]);

		// Act
		var result = await cmd.ExecuteBufferedAsync();

		Assume.That(result.ExitCode, Is.EqualTo(0));

		// Assert
		Assert.Multiple(() =>
		{
			Assert.That(result.StandardOutput.Trim(), Is.EqualTo("Hello stdout and stderr"));
			Assert.That(result.StandardError.Trim(), Is.EqualTo("Hello stdout and stderr"));
		});
	}
}