using Geekeey.Extensions.Process.Buffered;

namespace Geekeey.Extensions.Process.Tests;

[TestFixture]
internal sealed class PipingTests
{
	[Test]
	public async Task Pipe_StdinFromAsyncAnonymousSource()
	{
		// Arrange
		var source = PipeSource.Create(async (destination, cancellationToken)
			=> await destination.WriteAsync("Hello World!"u8.ToArray(), cancellationToken));

		var cmd = source | new Command(Dummy.Program.FilePath)
			.WithArguments("echo-stdin");

		// Act
		var result = await cmd.ExecuteBufferedAsync();

		// Assert
		Assert.That(result.StandardOutput.Trim(), Is.EqualTo("Hello World!"));
	}

	[Test]
	public async Task Pipe_StdinFromSyncAnonymousSource()
	{
		// Arrange
		var source = PipeSource.Create(destination
			=> destination.Write("Hello World!"u8.ToArray()));

		var cmd = source | new Command(Dummy.Program.FilePath)
			.WithArguments("echo-stdin");

		// Act
		var result = await cmd.ExecuteBufferedAsync();

		// Assert
		Assert.That(result.StandardOutput.Trim(), Is.EqualTo("Hello World!"));
	}

	[Test]
	public async Task Pipe_StdinFromStream()
	{
		// Arrange
		using var source = new MemoryStream("Hello World!"u8.ToArray());

		var cmd = source | new Command(Dummy.Program.FilePath)
			.WithArguments("echo-stdin");

		// Act
		var result = await cmd.ExecuteBufferedAsync();

		// Assert
		Assert.That(result.StandardOutput.Trim(), Is.EqualTo("Hello World!"));
	}

	[Test]
	public async Task Pipe_StdinFromMemory()
	{
		// Arrange
		var data = new ReadOnlyMemory<byte>("Hello World!"u8.ToArray());

		var cmd = data | new Command(Dummy.Program.FilePath)
			.WithArguments("echo-stdin");

		// Act
		var result = await cmd.ExecuteBufferedAsync();

		// Assert
		Assert.That(result.StandardOutput.Trim(), Is.EqualTo("Hello World!"));
	}

	[Test]
	public async Task Pipe_StdinFromByteArray()
	{
		// Arrange
		var data = "Hello World!"u8.ToArray();

		var cmd = data | new Command(Dummy.Program.FilePath)
			.WithArguments("echo-stdin");

		// Act
		var result = await cmd.ExecuteBufferedAsync();

		// Assert
		Assert.That(result.StandardOutput.Trim(), Is.EqualTo("Hello World!"));
	}

	[Test]
	public async Task Pipe_StdinFromString()
	{
		// Arrange
		var data = "Hello World!";

		var cmd = data | new Command(Dummy.Program.FilePath)
			.WithArguments("echo-stdin");

		// Act
		var result = await cmd.ExecuteBufferedAsync();

		// Assert
		Assert.That(result.StandardOutput.Trim(), Is.EqualTo("Hello World!"));
	}

	[Test]
	public async Task Pipe_NoStdinToProgramWhichExpectsInput()
	{
		// Arrange
		var cmd = new Command(Dummy.Program.FilePath)
			.WithArguments("echo-stdin");

		// Act
		await cmd.ExecuteAsync();
	}

	[Test]
	public async Task Pipe_EmptyDataToProgramWhichExpectsInput()
	{
		// Arrange
		var cmd = Array.Empty<byte>() | new Command(Dummy.Program.FilePath)
			.WithArguments("echo-stdin");

		// Act
		await cmd.ExecuteAsync();
	}
}