using Geekeey.Extensions.Process.Buffered;

namespace Geekeey.Extensions.Process.Tests;

[TestFixture]
internal sealed class ValidationTests
{
	[Test]
	public void Execute_Throws_OnNonZeroExit()
	{
		// Arrange
		var cmd = new Command(Dummy.Program.FilePath)
			.WithArguments(["exit", "1"]);

		// Act & Assert
		var exception = Assert.ThrowsAsync<CommandExecutionException>(async () => await cmd.ExecuteAsync());

		Assert.Multiple(() =>
		{
			Assert.That(exception.ExitCode, Is.EqualTo(1));
			Assert.That(exception.Message, Does.Contain("a non-zero exit code (1)"));
		});
	}

	[Test]
	public void ExecuteBuffered_Throws_OnNonZeroExit()
	{
		// Arrange
		var cmd = new Command(Dummy.Program.FilePath)
			.WithArguments(["exit", "1"]);

		// Act & Assert
		var exception = Assert.ThrowsAsync<CommandExecutionException>(async () => await cmd.ExecuteBufferedAsync());

		Assert.Multiple(() =>
		{
			Assert.That(exception.ExitCode, Is.EqualTo(1));
			Assert.That(exception.Message, Does.Contain("Exit code set to 1"));
		});
	}

	[Test]
	public async Task Execute_DoesNothingOnNone()
	{
		// Arrange
		var cmd = new Command(Dummy.Program.FilePath)
			.WithValidation(CommandExitBehaviour.None)
			.WithArguments(["exit", "1"]);

		// Act
		var exception = await cmd.ExecuteAsync();

		// Assert
		Assert.Multiple(() =>
		{
			Assert.That(exception.ExitCode, Is.EqualTo(1));
			Assert.That(exception.IsSuccess, Is.False);
		});
	}
}